
Imports System.IO
Imports System.Math

' Driver to calculate solar properties for hour, day or year
' Also reports radiation on 1 ft^2 panel facing south at a tilt of lat
' and radiation on a horz panel

Module SunPropertiesDrv
    Public Const RadPerDeg = 1 / 57.296
    Public Const DegPerRad = 57.296

    Sub Main()

        ' Solar properties for a particular day, month, hr
        Dim Day As Integer = 21
        Dim Month As Integer = 4
        Dim Year As Integer = 2001
        Dim AST As Double = 12
        Dim Longa As Double = 0
        Dim lat As Double = 45 / 57.3
        Dim Alt As Double = 5000
        Dim S As New SunCls(Day, Month, Year, AST, Longa, lat, Alt)
        S.Dump()

        Console.ReadLine()


        ' Output table of solar properties and radiation on a 1 ft^2 flat panel
        ' that is facing south at a tilt equal to lat

        Dim sw As New StreamWriter( _
                "c:\Documents and Settings\gary\My Documents\Visual Studio Projects\SunCls\out.txt")
        Dim Pn As VectorCls             ' Panel normal
        Dim Vsun As VectorCls           ' Vector toward sun

        Day = 21
        Month = 1
        Year = 2003
        AST = 12
        Longa = 0
        lat = 45 / DegPerRad
        Alt = 0
        Dim Tilt As Double = lat                                ' tilt of test panel
        Pn = VectorCls.AzElevVector(0, 90 * RadPerDeg - Tilt)   ' normal to test panel

        sw.WriteLine("Sun properties for year, and rad on test panel oriented S, and tilted Lat ")
        sw.WriteLine()

        Dim Sun1 As New SunCls(Day, Month, Year, AST, Longa, lat, Alt)
        Dim Az, Elev As Double
        Dim Idirnor As Double       ' direct solar radiation measure in direction of sun
        Dim Idirpnl As Double      ' direct component of radiation on panel corrected for incident angle
        Dim Idif As Double      ' diffuse componet of solar radiaiton
        Dim Idifpnl As Double    ' diffuse radiation on panel
        Dim Itot As Double      ' total radiation solar radiation 
        Dim Itotpnl As Double   ' total radiaion on panel
        Dim Ihorz As Double     ' radiation on horz panel
        Dim Icum, IcumYr As Double
        Dim Icumpnl, IcumpnlYr As Double
        Dim Icumhorz, IcumhorzYr As Double

        Dim ProjAngle As Double

        IcumYr = 0

        For Month = 1 To 12
            Icum = 0
            Icumpnl = 0
            Icumhorz = 0

            Sun1.Month = Month

            'find sunrise and sunset fot day
            Dim sr, ss As Double
            sr = Sun1.Sunrise
            sr = [Decimal].Floor(sr)
            ss = Sun1.Sunset
            ss = [Decimal].Floor(ss) + 1

            Dim hr As Double

            sw.WriteLine("Month Hr    Az  Elev Idirnor Idif   Itot |Idirpnl Idifpnl Itotpnl | Ihorz")


            For hr = sr To ss Step 1.0

                Sun1.ASThr = hr
                Az = Sun1.AzimuthAng            ' this will give + East az angles
                Elev = Sun1.AltAng

                If Elev > 0 Then

                    ' get a vector toward the sun
                    Vsun = Sun1.ToSunVector()

                    ' tilt angle of a prjectin of Vsun on a vertical plane oriented N, S
                    ProjAngle = Atan(Vsun.X / Vsun.Z)

                    ' get sun properties for this time
                    Idirnor = Sun1.Idn
                    Idif = Sun1.Idiffuse
                    Itot = Idirnor + Idif
                    Ihorz = Sun1.Ihorz()


                    ' calc radiation on South facing panel with Tilt = Lat
                    Idirpnl = Sun1.Idn * Vsun.Dot(Pn)
                    If Idirpnl < 0 Then Idirpnl = 0 ' sun coming from behind panel gives neg values for Idn
                    Idifpnl = Idif * (0.5 + 0.5 * Cos(Tilt))
                    Itotpnl = Idirpnl + Idifpnl

                    ' accumulations
                    Icum = Icum + Itot
                    Icumpnl = Icumpnl + Itotpnl
                    Icumhorz = Icumhorz + Ihorz

                    sw.WriteLine( _
                    "{0,4:N0}{1,5:N1}{2,5:N0}{3,5:N0} {4,6:N0} {5,6:N0} {6,6:N0} | {7,6:N0} {8,6:N0}  {9,6:N0}  |{10,6:N0} {11,6:N1}", _
                            Month, hr, Az * DegPerRad, Elev * DegPerRad, _
                            Idirnor, Idif, Itot, _
                            Idirpnl, Idifpnl, Itotpnl, Ihorz, ProjAngle * DegPerRad)

                End If

            Next

            sw.WriteLine("  Month    Day    Icum    Icumpnl   Icumhorz")
            sw.WriteLine("{0,6:D} {1,6:D}   {2,6:N0}     {3,6:N0}    {4,6:N0}", _
                            Month, Day, Icum, Icumpnl, Icumhorz)
            sw.WriteLine()
            IcumYr = IcumYr + Icum * 30
            IcumpnlYr = IcumpnlYr + Icumpnl * 30
            IcumhorzYr = IcumhorzYr + Icumhorz * 30
        Next

        sw.WriteLine()
        sw.WriteLine("APROXIMATE  IcumYr        = {0,10:N0}", IcumYr)
        sw.WriteLine("APROXIMATE  Icum Panel Yr = {0,10:N0}", IcumpnlYr)
        sw.WriteLine("APROXIMATE  Icum Horz  Yr = {0,10:N0}", IcumhorzYr)
        sw.Close()

    End Sub

End Module
